"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var minimist_1 = __importDefault(require("minimist"));
var xterm_1 = require("xterm");
var fit_1 = require("xterm/lib/addons/fit/fit");
var Cli = /** @class */ (function () {
    function Cli() {
        var _this = this;
        this.commands = {};
        this.buffer = '';
        this.cursorOffset = 0;
        this.terminalPrompt = '> ';
        this.terminalHistory = [];
        this.terminalHistoryIndex = 0;
        this.terminal = new xterm_1.Terminal({
            cursorBlink: true,
            cursorStyle: 'underline'
        });
        this.terminal.on('key', function (d) { return _this.input(d); });
        this.terminal.on('paste', function (d) { return _this.input(d); });
    }
    Cli.prototype.input = function (str) {
        for (var i = 0; i < str.length; i++) {
            //console.log(str.charCodeAt(i))
            switch (str.charCodeAt(i)) {
                case 8: // backspace
                    if (this.cursorOffset === 0)
                        break;
                    this.cursorOffset--;
                    this.buffer = this.buffer.substr(0, this.cursorOffset) + this.buffer.substr(this.cursorOffset + 1);
                    this.terminal.write('\b \b');
                    break;
                case 127: // del
                    if (this.cursorOffset === 0)
                        break;
                    // TODO: change it so it works outside browser too
                    if (navigator.platform.indexOf('Mac') >= 0) {
                        this.cursorOffset--;
                        this.buffer = this.buffer.substr(0, this.cursorOffset) + this.buffer.substr(this.cursorOffset + 1);
                        this.terminal.write('\b \b');
                    }
                    break;
                case 13: // enter
                    this.parseInput();
                    this.prompt();
                    break;
                case 65: // up
                    var cmd = this.terminalHistory[this.terminalHistoryIndex - 1];
                    if (cmd) {
                        this.terminalHistoryIndex--;
                        this.terminal.write(Array(this.buffer.length + 1).join('\b \b'));
                        this.terminal.write(cmd);
                        this.cursorOffset = cmd.length;
                        this.buffer = cmd;
                    }
                    break;
                case 66: // down
                    var cmd = this.terminalHistory[this.terminalHistoryIndex + 1];
                    if (cmd) {
                        this.terminalHistoryIndex++;
                        this.terminal.write(Array(this.buffer.length + 1).join('\b \b'));
                        this.terminal.write(cmd);
                        this.cursorOffset = cmd.length;
                        this.buffer = cmd;
                    }
                    break;
                case 67: // right
                case 68: // left
                    if (str.charCodeAt(i) === 67)
                        this.cursorOffset++;
                    if (str.charCodeAt(i) === 68)
                        this.cursorOffset--;
                    this.terminal.write(String.fromCharCode(27) + String.fromCharCode(91) + str[i]);
                    break;
                case 27:
                case 91:
                    // skip
                    break;
                default:
                    var rest = this.buffer.substr(this.cursorOffset);
                    this.buffer = this.buffer.substr(0, this.cursorOffset) + str[i] + rest;
                    this.cursorOffset++;
                    this.terminal.write(str[i] + rest + Array(rest.length + 1).join('\b'));
                    break;
            }
        }
    };
    Cli.prototype.parseInput = function () {
        var buffer = this.buffer.trim();
        if (buffer) {
            var args = minimist_1.default(buffer.split(' '));
            var cmdName = args._[0];
            var cmd = this.commands[cmdName];
            if (cmd) {
                var res = cmd.action();
                var output = JSON.stringify(res, null, 2).replace(/\r?\n/g, '\r\n');
                this.terminal.write("\r\n" + output);
            }
            else {
                this.showHelp();
            }
            this.terminalHistory.push(buffer);
            this.terminalHistoryIndex = this.terminalHistory.length;
        }
        this.cursorOffset = 0;
        this.buffer = '';
    };
    Cli.prototype.prompt = function (newLine) {
        if (newLine === void 0) { newLine = true; }
        this.terminal.write("" + (newLine ? '\r\n' : '') + this.terminalPrompt);
    };
    Cli.prototype.showHelp = function () {
    };
    Cli.prototype.register = function (command) {
        this.commands[command.name] = command;
    };
    Cli.prototype.show = function (domElement) {
        this.terminal.open(domElement);
        this.prompt(false);
        fit_1.fit(this.terminal);
    };
    return Cli;
}());
exports.Cli = Cli;
